<?php

namespace Drupal\preview_site\Commands;

use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\preview_site\Entity\PreviewSiteBuildInterface;
use Drupal\preview_site\PreviewSiteBuilder;
use Drush\Commands\DrushCommands;
use Psr\Log\LogLevel;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Preview site drush commands.
 */
class PreviewSiteCommands extends DrushCommands {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Builder.
   *
   * @var \Drupal\preview_site\PreviewSiteBuilder
   */
  protected $builder;

  /**
   * State.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Creates a PreviewSiteCommands object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\DependencyInjection\ClassResolverInterface $class_resolver
   *   The class resolver service.
   * @param \Drupal\workflows\StateInterface $state
   *   The state service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ClassResolverInterface $class_resolver, StateInterface $state) {
    parent::__construct();
    $this->builder = $class_resolver->getInstanceFromDefinition(PreviewSiteBuilder::class);
    $this->entityTypeManager = $entity_type_manager;
    $this->state = $state;
  }

  /**
   * Provide a list of preview sites if argument is not provided.
   *
   * @hook interact preview-site:deploy
   */
  public function previewSiteDeployInteract(InputInterface $input, OutputInterface $output) {
    if (!empty($input->getArgument('preview_site_id'))) {
      return;
    }

    $preview_site_builds = $this->entityTypeManager->getStorage('preview_site_build')->loadMultiple();
    $choices = [];
    foreach ($preview_site_builds as $preview_site_build) {
      // Choices does not support integer keys, so do some string manipulation.
      $choices[(string) 'id_' . $preview_site_build->id()] = $preview_site_build->label();
    }
    $choice = $this->io()->choice(dt('Which preview site would you like to deploy'), $choices);

    $input->setArgument('preview_site_id', (int) substr($choice, 3));
  }

  /**
   * Deploy a preview site by ID.
   *
   * @param int $preview_site_id
   *   The preview_site_build entity id to rebuild.
   *
   * @command preview-site:deploy
   *
   * @option clear-lock
   *   Clear the build lock.
   *
   * @validate-module-enabled preview_site
   */
  public function previewSiteDeploy(int $preview_site_id, array $options = ['clear-lock' => FALSE]) {
    $preview_site_build = $this->entityTypeManager->getStorage('preview_site_build')->load($preview_site_id);

    if (empty($preview_site_build)) {
      $message = dt('Error loading preview site with ID :id', [
        ':id' => $preview_site_id,
      ]);
      $this->logger()->log($message, LogLevel::ERROR);
      $this->output()->writeln($message);
      return;
    }

    if ($options['clear-lock']) {
      $this->state->delete(PreviewSiteBuildInterface::BUILDING_STATE_KEY);
    }

    batch_set($this->builder->getPreviewSiteBuildBatch($preview_site_build)->toArray());
    drush_backend_batch_process();
  }

}
